/*

    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>



#define VER     "0.1"
#define BUFFSZ  32768



typedef unsigned char   u_char;



void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd,
            *fdout;
    int     len;
    u_char  buff[BUFFSZ],
            *ptr,
            *limit;


    setbuf(stdout, NULL);

    fputs("\n"
        "Uplink users and data.dat files decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\n"
            "Usage: %s <file.USR> <output_file>\n"
            "\n"
            "Note: users files are in the directory users, then you must use an hex editor\n"
            "      to retrieve the needed data\n"
            "Note: this tool works also versus data.dat and after decoding it you can\n"
            "      modify it and don't need to re-encode it.\n"
            "      Instead world.dat seems to be only a file that the program checks at\n"
            "      runtime but doesn't contain needed data so don't decode it\n"
            "\n", argv[0]);
        exit(1);
    }

    printf("- Open input file:    %s\n", argv[1]);
    fd = fopen(argv[1], "rb");
    if(!fd) std_err();

    printf("- Open output file:   %s\n", argv[2]);
    fdout = fopen(argv[2], "wb");
    if(!fdout) std_err();

    fread(buff, 9, 1, fd);
    if(memcmp(buff, "REDSHIRT\0", 9)) {
        fputs("\n"
            "Error: the input file is not an Uplink encoded file\n"
            "\n", stdout);
        exit(1);
    }

    while((len = fread(buff, 1, BUFFSZ, fd))) {
        ptr = buff;
        limit = buff + len;
        do {
            *ptr += 0x80;
        } while(++ptr != limit);

        fwrite(buff, len, 1, fdout);
    }

    fclose(fd);
    fclose(fdout);
    fputs("- finished\n", stdout);

    return(0);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}

